<?php

namespace App\Providers;

use App\Models\Currency;
use App\Models\Language;
use Illuminate\
    {
        Support\Facades\DB,
        Support\Collection,
        Support\ServiceProvider,
        Pagination\LengthAwarePaginator
    };

use App\Models\Font;
use Illuminate\Pagination\Paginator;
use Illuminate\Support\Facades\Session;

class AppServiceProvider extends ServiceProvider
{
    public function boot()
    {
        Paginator::useBootstrap();
        \Cache::flush('generalsettings');


        view()->composer('*',function($settings){

            $settings->with('gs', cache()->remember('generalsettings', now()->addDay(), function () {
                return DB::table('generalsettings')->first();
            }));


            $settings->with('langg', cache()->remember('generalsettings', now()->addDay(), function () {
                return DB::table('generalsettings')->first();
            }));

            $settings->with('ps', cache()->remember('pagesettings', now()->addDay(), function () {
                return DB::table('pagesettings')->first();
            }));

            $settings->with('seo', cache()->remember('seotools', now()->addDay(), function () {
                return DB::table('seotools')->first();
            }));
            $settings->with('socialsetting', cache()->remember('socialsettings', now()->addDay(), function () {
                return DB::table('socialsettings')->first();
            }));

            $settings->with('default_font', cache()->remember('default_font', now()->addDay(), function () {
                return Font::whereIsDefault(1)->first();
            }));

            if (Session::has('currency'))
            {
                $settings->with('curr',  cache()->remember('session_currency', now()->addDay(), function () {
                    return Currency::find(Session::get('currency'));
                }));
            }
            else
            {
                $settings->with('curr', cache()->remember('default_currency', now()->addDay(), function () {
                    return Currency::where('is_default','=',1)->first();
                }));
            }

             if (Session::has('language'))
            {
                $settings->with('langg',  cache()->remember('session_language', now()->addDay(), function () {
                    return Language::find(Session::get('language'));
                }));
            }
            else
            {
                $settings->with('langg', cache()->remember('session_language', now()->addDay(), function () {
                    return Language::where('is_default','=',1)->first();
                }));
            }
        });
    }

    public function register()
    {
        Collection::macro('paginate', function($perPage, $total = null, $page = null, $pageName = 'page') {
            $page = $page ?: LengthAwarePaginator::resolveCurrentPage($pageName);
            return new LengthAwarePaginator(
                $this->forPage($page, $perPage),
                $total ?: $this->count(),
                $perPage,
                $page,
                [
                    'path' => LengthAwarePaginator::resolveCurrentPath(),
                    'pageName' => $pageName,
                ]
            );
        });
    }
}
